@@ -1,12 +1,14 @@
 # Copyright (c) Microsoft Corporation.
 # Licensed under the MIT License
 import os
+
 import pytest
 
+from azure.kusto.data._cloud_settings import CloudInfo
 from azure.kusto.data._decorators import aio_documented_by
 from azure.kusto.data._token_providers import *
 from .test_kusto_client import run_aio_tests
-from ..test_token_providers import KUSTO_URI, PUBLIC_AUTH_URI, TOKEN_VALUE, TEST_AZ_AUTH, TEST_MSI_AUTH, TEST_DEVICE_AUTH, TokenProviderTests
+from ..test_token_providers import KUSTO_URI, TOKEN_VALUE, TEST_AZ_AUTH, TEST_MSI_AUTH, TEST_DEVICE_AUTH, TokenProviderTests
 
 
 class MockProvider(TokenProviderBase):
@@ -19,7 +21,7 @@ class MockProvider(TokenProviderBase):
     def name() -> str:
         return "MockProvider"
 
-    def context(self) -> dict:
+    def _context_impl(self) -> dict:
         return {"authority": "MockProvider"}
 
     def _init_impl(self):
@@ -179,7 +181,7 @@ class TestTokenProvider:
         auth = os.environ.get("USER_AUTH_ID", "organizations")
 
         if username and password and auth:
-            provider = UserPassTokenProvider(KUSTO_URI, PUBLIC_AUTH_URI + auth, username, password)
+            provider = UserPassTokenProvider(KUSTO_URI, auth, username, password)
             token = await provider.get_token_async()
             assert self.get_token_value(token) is not None
 
@@ -200,7 +202,7 @@ class TestTokenProvider:
             # break here if you debug this test, and get the code from 'x'
             print(x)
 
-        provider = DeviceLoginTokenProvider(KUSTO_URI, PUBLIC_AUTH_URI + "organizations", callback)
+        provider = DeviceLoginTokenProvider(KUSTO_URI, "organizations", callback)
         token = await provider.get_token_async()
         assert self.get_token_value(token) is not None
 
@@ -218,7 +220,7 @@ class TestTokenProvider:
         app_key = os.environ.get("APP_KEY")
 
         if app_id and app_key and auth_id:
-            provider = ApplicationKeyTokenProvider(KUSTO_URI, PUBLIC_AUTH_URI + auth_id, app_id, app_key)
+            provider = ApplicationKeyTokenProvider(KUSTO_URI, auth_id, app_id, app_key)
             token = await provider.get_token_async()
             assert self.get_token_value(token) is not None
 
@@ -243,7 +245,7 @@ class TestTokenProvider:
             with open(pem_key_path, "rb") as file:
                 pem_key = file.read()
 
-            provider = ApplicationCertificateTokenProvider(KUSTO_URI, cert_app_id, PUBLIC_AUTH_URI + cert_auth, pem_key, thumbprint)
+            provider = ApplicationCertificateTokenProvider(KUSTO_URI, cert_app_id, cert_auth, pem_key, thumbprint)
             token = await provider.get_token_async()
             assert self.get_token_value(token) is not None
 
@@ -255,7 +257,7 @@ class TestTokenProvider:
                 with open(public_cert_path, "r") as file:
                     public_cert = file.read()
 
-                provider = ApplicationCertificateTokenProvider(KUSTO_URI, cert_app_id, PUBLIC_AUTH_URI + cert_auth, pem_key, thumbprint, public_cert)
+                provider = ApplicationCertificateTokenProvider(KUSTO_URI, cert_app_id, cert_auth, pem_key, thumbprint, public_cert)
                 token = await provider.get_token_async()
                 assert self.get_token_value(token) is not None
 
@@ -267,3 +269,47 @@ class TestTokenProvider:
 
         else:
             print(" *** Skipped App Cert Provider Test ***")
+
+    @aio_documented_by(TokenProviderTests.test_cloud_mfa_off)
+    @pytest.mark.asyncio
+    async def test_cloud_mfa_off(self):
+        FAKE_URI = "https://fake_cluster_for_login_mfa_test.kusto.windows.net"
+        cloud = CloudInfo(
+            login_endpoint="https://login_endpoint",
+            login_mfa_required=False,
+            kusto_client_app_id="1234",
+            kusto_client_redirect_uri="",
+            kusto_service_resource_id="https://fakeurl.kusto.windows.net",
+            first_party_authority_url="",
+        )
+        CloudSettings._cloud_cache[FAKE_URI] = cloud
+        authority = "auth_test"
+
+        provider = UserPassTokenProvider(FAKE_URI, authority, "a", "b")
+        await provider._init_once_async(init_only_cloud=True)
+        context = provider.context()
+        assert context["authority"] == "https://login_endpoint/auth_test"
+        assert context["client_id"] == "1234"
+        assert provider._scopes == ["https://fakeurl.kusto.windows.net/.default"]
+
+    @aio_documented_by(TokenProviderTests.test_cloud_mfa_off)
+    @pytest.mark.asyncio
+    async def test_cloud_mfa_on(self):
+        FAKE_URI = "https://fake_cluster_for_login_mfa_test.kusto.windows.net"
+        cloud = CloudInfo(
+            login_endpoint="https://login_endpoint",
+            login_mfa_required=True,
+            kusto_client_app_id="1234",
+            kusto_client_redirect_uri="",
+            kusto_service_resource_id="https://fakeurl.kusto.windows.net",
+            first_party_authority_url="",
+        )
+        CloudSettings._cloud_cache[FAKE_URI] = cloud
+        authority = "auth_test"
+
+        provider = UserPassTokenProvider(FAKE_URI, authority, "a", "b")
+        await provider._init_once_async(init_only_cloud=True)
+        context = provider.context()
+        assert context["authority"] == "https://login_endpoint/auth_test"
+        assert context["client_id"] == "1234"
+        assert provider._scopes == ["https://fakeurl.kustomfa.windows.net/.default"]
